from typing import Any

class MiniBatchKMeans:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

    inertia_: Any
    cluster_centers_: Any

    def fit(self, X: Any, y: Any = ...) -> "MiniBatchKMeans": ...
    def partial_fit(self, X: Any, y: Any = ...) -> "MiniBatchKMeans": ...
    def predict(self, X: Any) -> Any: ...
    def fit_predict(self, X: Any, y: Any = ...) -> Any: ...
